@@ -1,3 +1,6 @@
+# docs/COPYING 2a + DRY: https://github.com/getmail6/getmail6
+# Please refer to the git history regarding who changed what and when in this file.
+
 '''The getmail Message class.
 
 '''
@@ -128,7 +131,8 @@ class Message(object):
             # Can't happen?
             raise SystemExit('Message() called with wrong arguments')
 
-        self.sender = address_no_brackets(self.__msg['return-path']
+        self.sender = address_no_brackets(self.__msg['Return-Path']
+                                          or self.__msg['Sender']
                                           or 'unknown')
 
     def content(self):
